/******************************************************************************
    DETETOR DE METAIS PI WHITE SURF LPI  V1

    c/ PIC16F675


    Autor: Cludio Lrios

    DATA: 28/05/2024

    Objetivo: Substituir os circuitos integrados 4011 (4001), NE555 (2) do
    detetor de metais PI WHITE SURFMASTER original por um PIC 12F675(629)
    Ajustado para produzir unicamente os pulsos:
    Pino 3 -> pulso negativo de 50 useg para acionamento do transistor da
              bobina sensora(FREQ 600HZ).
    Pino 5 -> Potencimetro 1
    Pino 2 -> pulso negativo de 40 useg para o FET BF245 amostragem do pulso
    Pino 6 -> pulso negativo de 40 useg para o FET BF245 amostragem da tenso mdia
    Pino 7 -> pulso positivo de 140 useg (entre pulsos do pino 2 e 6)

    PINAGEM DO PIC 12F675:
    PINO 7 = GP0  sada para gerao de som
    PINO 6 = GP1  sada para o transistor Fet (ATM)
    PINO 5 = GP2  sada para o transistor Fet (AP)
    PINO 4 = GP3  reset
    PINO 3 = GP4  sada para o transistor driver da bobina
    PINO 2 = GP5  potencimetro (DISC)


    Uso didtico

    Blog Picsource
******************************************************************************/

#include <12F675.h>
#use delay(clock=4000000)//OSC INTERNO
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, MCLR, NOPUT, NOBROWNOUT /*INTRC*/


//bytes

#byte   FSR         = 0x04
#byte   PCLATH      = 0x0A
#byte   STATUS      = 0X03
#byte   TRISIO      = 0x85
#byte   GPIO        = 0x05
#byte   TMR1H       = 0x0F
#byte   TMR1L       = 0x0E
#byte   TMR0        = 0X01
#byte   OPTION_REG  = 0x81
#byte   WPU         = 0x95
#byte   CMCON       = 0x19
#byte   ANSEL       = 0x9F
#byte   ADRESH       = 0x1E
#byte   ADRESL       = 0x9E
#byte   T1CON       = 0X10
#byte   IOC         = 0x96
#byte   INTCON      = 0X0B
#byte   VRCON       = 0X99
#byte   ADCON0      = 0X1F
#byte   VRCON       = 0X99
#byte   FSR         = 0X04
#byte   INDF        = 0X00

//bit
#bit    GIE         = 0xb.7
#bit    PEIE        = 0xb.6
#bit    T0IE        = 0xb.5
#bit    TMR1IE      = 0x8C.0
#bit    TMR1IF      = 0xC.0
#bit    TMR1_ON     = 0x10.0
#bit    T0IF        = 0XB.2
#bit    TMR0_F0     = 0x01.0
#bit    CMCOM_F6    = 0x19.6
#bit    GO_DONE     = 0X1F.1
#bit    COUT        = 0X19.6

//pinos
//#bit out_aux     = 0x5.2  //pino 5 - entrada potenciometro DISC
#bit out_mosfet  = 0x5.5  //pino 2 - Acionamento do Mosfet
#bit out_fetap   = 0x5.4  //pino 3 - Acionamento do Fet  AP
#bit out_fetatm  = 0x5.1  //pino 6 - Acionamento do Fet ATM

#bit out_som     = 0x5.0  //pino 7 - Acionamento do som


//=============================================================================
//   INICIALIZAO DE PORTS E REGISTRADORES
//=============================================================================
void main (){


  //-----------------------------------------------------
  // PIC 12F675  definies de portas e registradores

  OPTION_REG=0;
  WPU=0;//sem pullups
  ADCON0 = 0B00001001; //JUST. ESQU/REF=VDD/CANAL2
  ANSEL = 0B00010100;   // osc/8 / canal AN2 = analgico
  TRISIO = 0b001100;  // entradas e sadas
  CMCON =  7;  // desliga comparador interno
  GPIO =   0b010110;  // desliga sada


//=============================================================================
//   LOOP PRINCIPAL DE REPETIO DO PROGRAMA
//=============================================================================
  for(;;){
  char a,pdelay;
  int16 soma;

  #define  TEMPO_MOSFET_ON  60
  #define  TEMPO_INTERVALO  40
  #define  TEMPO_FET_AP     40
  #define  TEMPO_FET_ATM    40
  #define  TEMPO_GER_SOM   140


  soma=0;
  for(a=0;a<8;a++){
    GO_DONE=1;
    while(GO_DONE==1);
    soma= soma + ADRESH;
  }
  soma= soma/128;
  pdelay=soma;
 
  if(pdelay==0)pdelay=1;

  out_mosfet=1;//inicia pulso no MosFet
  delay_us(TEMPO_MOSFET_ON);//delay da largura do pulso do MosFet
  out_mosfet=0;//finaliza pulso no MosFet

//delay do intervalo no final de pulso para o do Fet AP

  do
   pdelay--;
  while(pdelay);

  out_som=1;//liga som
  out_fetap=1;//aciona Fet para capturar leitura (invertido)
  delay_us(TEMPO_FET_AP );//janela de tempo que fica capturando a leitura
  out_fetap=0;//desliga FET e termina a janela de captura
 // out_som=1;//liga som
  delay_us(TEMPO_GER_SOM);//delay complementar
  out_som=0;//desliga som
  out_fetatm=1;//liga (invertido)
  delay_us(TEMPO_FET_ATM);
  out_fetatm=0;//desliga (invertido)




/* Delay para gerar a frequncia de oscilao da bobina e tambm gerando os pulsos
da fonte auxiliar (freq ~= 5400 Hz) */
  for(a=0;a<5;a++){//49 para ~100hz / 12 para ~400 hz
   delay_us(200);
  }

  }//for(;;)
}//void main()
//-----------------------------------------------------------------------------


